/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __geometry_PointCurveNearest_h__
#define __geometry_PointCurveNearest_h__

namespace fe
{

/**************************************************************************//**
	@brief Find point nearest to a curve with radius

	@ingroup geometry
*//***************************************************************************/
template <typename T>
class PointCurveNearest
{
	public:
static	T	solve(const Vector<3,T>* a_pVertex,const U32 a_vertCount,
					const T a_radius,
					const Vector<3,T>& a_origin,Vector<3,T>& a_direction,
					T& a_along,Vector<3,T>& a_intersection);
};

template <typename T>
inline T PointCurveNearest<T>::solve(
	const Vector<3,T>* a_pVertex,const U32 a_vertCount,
	const T a_radius,
	const Vector<3,T>& a_origin, Vector<3,T>& a_direction,
	T& a_along,Vector<3,T>& a_intersection)
{
	T bestMag= -1.0;
	for(U32 m=0;m<a_vertCount-1;m++)
	{
		SpatialVector oneIntersection;
		T oneAlong;
		T oneMag=PointCylinderNearest<T>::solve(a_pVertex[m],
				unit(a_pVertex[m+1]-a_pVertex[m]),a_radius,
				a_origin,a_direction,oneAlong,oneIntersection);
		if(oneMag>=0.0 && (bestMag<0.0 || oneMag<bestMag))
		{
			a_intersection=oneIntersection;
			a_along=oneAlong;
			bestMag=oneMag;
		}
	}

	if(bestMag>=0.0)
	{
		a_along=(a_vertCount-1+a_along)/a_vertCount;
	}

	return bestMag;
}

} /* namespace */

#endif /* __geometry_PointCurveNearest_h__ */
