/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "houdini/houdini.pmh"

namespace fe
{

void SurfaceAccessibleHoudini::append(
	sp<SurfaceAccessibleI> a_spSurfaceAccessibleI,
	SpatialTransform& a_rTransform)
{
	SAFEGUARD;

	feLog("SurfaceAccessibleHoudini::append\n");
	sp<SurfaceAccessibleHoudini> spSurfaceOther=
			a_spSurfaceAccessibleI;
	if(!spSurfaceOther.isValid())
	{
		//* TODO support cross copies
		feLog("SurfaceAccessibleHoudini::append"
				" not SurfaceAccessibleHoudini\n");
		return;
	}

	UT_Matrix4 utMatrix4(
			a_rTransform(0,0),a_rTransform(1,0),a_rTransform(2,0),0.0,
			a_rTransform(0,1),a_rTransform(1,1),a_rTransform(2,1),0.0,
			a_rTransform(0,2),a_rTransform(1,2),a_rTransform(2,2),0.0,
			a_rTransform(0,3),a_rTransform(1,3),a_rTransform(2,3),1.0);

	const GU_Detail* pOtherGdp=spSurfaceOther->gdp();
	GU_Detail copyGdp;
	copyGdp.duplicate(*pOtherGdp);
	copyGdp.transform(utMatrix4);
	const_cast<GU_Detail*>(m_pGdp)->merge(copyGdp);
}

void SurfaceAccessibleHoudini::instance(sp<SurfaceAccessibleI>
	a_spSurfaceAccessibleI,std::vector<SpatialTransform>& a_rTransformArray)
{
	//* TODO
}

sp<SurfaceI> SurfaceAccessibleHoudini::surface(void)
{
	return surface(String());
}

sp<SurfaceI> SurfaceAccessibleHoudini::surface(String a_group)
{
	if(!m_pGdp)
	{
		return sp<SurfaceI>(NULL);
	}

	SAFEGUARD;
	if(m_spSurfaceI.isNull())
	{
		const GEO_Primitive* pPrimitive=m_pGdp->primitives()(0);

		if(m_pGdp->primitives().entries()==1)
		{
			//* single primitive

#ifdef FE_HOUDINI_USE_GA
			if(pPrimitive && pPrimitive->getTypeId().get()==GEO_PRIMSPHERE)
#else
			if(pPrimitive && pPrimitive->getPrimitiveId()==GEOPRIMSPHERE)
#endif
			{
				//* single sphere
				return subSurface(0,a_group);
			}
		}

		//* HACK presuming they're all the same
		const GEO_Face* pGeoFace=dynamic_cast<const GEO_Face*>(pPrimitive);
		const BWORD openPoly=pGeoFace && !pGeoFace->isClosed();
		if(openPoly)
		{
			//* HACK presuming uniform CV counts
			const U32 subCount=pGeoFace->getVertexCount();

			//* all curves
			m_spSurfaceI=registry()->create("SurfaceI.SurfaceCurvesHoudini");
			if(m_spSurfaceI.isValid())
			{
				sp<SurfaceCurvesHoudini> spSurface=m_spSurfaceI;
				if(spSurface.isValid())
				{
					spSurface->setGdp(m_pGdp);
					spSurface->setGroup(a_group);
					spSurface->setGrain(subCount);
					spSurface->setOpenCurve(TRUE);
				}
			}
		}

		//* all triangles (presumed)
		m_spSurfaceI=registry()->create("SurfaceI.SurfaceTrianglesHoudini");
		if(m_spSurfaceI.isValid())
		{
			sp<SurfaceTrianglesHoudini> spSurface=m_spSurfaceI;
			if(spSurface.isValid())
			{
				spSurface->setGdp(m_pGdp);
				spSurface->setGroup(a_group);
			}
		}
	}

	return m_spSurfaceI;
}

sp<SurfaceI> SurfaceAccessibleHoudini::subSurface(U32 a_subIndex)
{
	return subSurface(a_subIndex,String());
}

sp<SurfaceI> SurfaceAccessibleHoudini::subSurface(U32 a_subIndex,String a_group)
{
	if(!m_pGdp)
	{
		return sp<SurfaceI>(NULL);
	}

	SAFEGUARD;
	const GEO_Primitive* pPrimitive=m_pGdp->primitives()(a_subIndex);
	if(!pPrimitive)
	{
		return sp<SurfaceI>(NULL);
	}

#ifdef FE_HOUDINI_USE_GA
	if(pPrimitive->getTypeId().get()==GEO_PRIMSPHERE)
#else
	if(pPrimitive->getPrimitiveId()==GEOPRIMSPHERE)
#endif
	{
		sp<SurfaceSphere> spSphere=
				registry()->create("SurfaceI.SurfaceSphere");
		if(!spSphere.isValid())
		{
			return sp<SurfaceI>(NULL);
		}

		const GEO_PrimSphere* pPrimSphere=
				dynamic_cast<const GEO_PrimSphere*>(pPrimitive);

		UT_Matrix4 utMatrix4;
		pPrimSphere->getTransform4(utMatrix4);
		SpatialTransform matrix;
		set(matrix,utMatrix4.data());
//		feLog("SurfaceAccessibleHoudini::subSurface(%d) sphere matrix\n%s\n",
//				a_subIndex,c_print(matrix));

		spSphere->setCenter(matrix.translation());
		spSphere->setRadius(matrix(0,0));

		return spSphere;
	}

	sp<SurfaceTrianglesHoudini> spTriangles=
			registry()->create("SurfaceI.SurfaceTrianglesHoudini");
	if(!spTriangles.isValid())
	{
		return sp<SurfaceI>(NULL);
	}

	spTriangles->setGdp(m_pGdp);
	spTriangles->setSubIndex(a_subIndex);
	spTriangles->setGroup(a_group);
	return spTriangles;
}

} // namespace
