/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "houdini/houdini.pmh"

namespace fe
{

void SurfaceAccessorHoudini::addString(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addString"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}
void SurfaceAccessorHoudini::addInteger(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addInteger"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addReal(void)
{
//	feLog("SurfaceAccessorHoudini::addReal \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		const Real zero=0.0;
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addReal"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addSpatialVector(void)
{
//	feLog("SurfaceAccessorHoudini::addSpatialVector \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 3);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#if FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 3);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addSpatialVector"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

BWORD SurfaceAccessorHoudini::bindInternal(
	SurfaceAccessibleI::Element a_element,const String& a_name)
{
	FEASSERT(m_pGdp);
	m_element=SurfaceAccessibleI::e_point;
	if(a_element>=0 && a_element<=5)
	{
		m_element=a_element;
	}
	m_name=a_name;
	m_handle.invalidate();
	m_pPoint=NULL;
	m_pPointChangeable=NULL;
	m_pPrimitive=NULL;
	m_pGroup=NULL;
	m_groupIndex=0;
	switch(m_element)
	{
		case SurfaceAccessibleI::e_point:
			if(m_pGdpChangeable)
			{
				m_handle=m_pGdpChangeable->getPointAttribute(a_name.c_str());
			}
			else
			{
				m_handle=m_pGdp->getPointAttribute(a_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_pointGroup:
			if(m_pGdpChangeable)
			{
				m_pGroup=m_pGdpChangeable->findPointGroup(m_name.c_str());
			}
			else
			{
				m_pGroup=m_pGdp->findPointGroup(m_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_vertex:
			break;

		case SurfaceAccessibleI::e_primitive:
			if(m_pGdpChangeable)
			{
				m_handle=m_pGdpChangeable->getPrimAttribute(a_name.c_str());
			}
			else
			{
				m_handle=m_pGdp->getPrimAttribute(a_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_primitiveGroup:
			if(m_pGdpChangeable)
			{
				m_pGroup=m_pGdpChangeable->findPrimitiveGroup(m_name.c_str());
			}
			else
			{
				m_pGroup=m_pGdp->findPrimitiveGroup(m_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_detail:
			break;

		default:
			;
	}
	return isBound();
}

void SurfaceAccessorHoudini::select(U32 index)
{
	FEASSERT(m_pGdp);
	switch(m_element)
	{
		case SurfaceAccessibleI::e_point:
			if(m_pGdpChangeable)
			{
#ifdef FE_HOUDINI_USE_GA
				m_pPointChangeable=m_pGdpChangeable->getGEOPoint(index);
#else
				m_pPointChangeable=m_pGdpChangeable->points()(index);
#endif
				m_handle.setElement(m_pPointChangeable);
				m_pPoint=m_pPointChangeable;
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pPoint=m_pGdp->getGEOPoint(index);
#else
				m_pPoint=m_pGdp->points()(index);
#endif
				m_handle.setElement(m_pPoint);
				m_pPointChangeable=NULL;
			}
			break;

		case SurfaceAccessibleI::e_primitive:
			if(m_pGdpChangeable)
			{
#ifdef FE_HOUDINI_USE_GA
				GEO_Primitive* pPrimitive=
						m_pGdpChangeable->getGEOPrimitive(index);
#else
				GEO_Primitive* pPrimitive=
						m_pGdpChangeable->primitives()(index);
#endif
				m_handle.setElement(pPrimitive);
				m_pPrimitive=pPrimitive; //* to const
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pPrimitive=m_pGdp->getGEOPrimitive(index);
#else
				m_pPrimitive=m_pGdp->primitives()(index);
#endif
				m_handle.setElement(m_pPrimitive);
			}
			break;

		case SurfaceAccessibleI::e_pointGroup:
		{
#ifdef FE_HOUDINI_USE_GA
			GA_PointGroup& rPointGroup=*(GA_PointGroup*)m_pGroup;
#else
			GB_PointGroup& rPointGroup=*(GB_PointGroup*)m_pGroup;
#endif
			if(m_pGdpChangeable)
			{
				if(!m_pPointChangeable)
				{
					m_groupIndex=0;
					m_pPointChangeable=
							m_pGdpChangeable->points().head(rPointGroup);
				}
				U32 m=m_groupIndex;
				for(;m>index && m_pPointChangeable;m--)
				{
					m_pPointChangeable=
							m_pGdpChangeable->points().prev(
							m_pPointChangeable,rPointGroup);
				}
				for(;m<index && m_pPointChangeable;m++)
				{
					m_pPointChangeable=
							m_pGdpChangeable->points().next(
							m_pPointChangeable,rPointGroup);
				}
				m_groupIndex=m;
				m_pPoint=m_pPointChangeable;
			}
			else
			{
				m_pPointChangeable=NULL;

				if(!m_pPoint)
				{
					m_groupIndex=0;
					m_pPoint=m_pGdp->points().head(rPointGroup);
				}
				U32 m=m_groupIndex;
				for(;m>index && m_pPoint;m--)
				{
					m_pPoint=m_pGdp->points().prev(m_pPoint,rPointGroup);
				}
				for(;m<index && m_pPoint;m++)
				{
					m_pPoint=m_pGdp->points().next(m_pPoint,rPointGroup);
				}
				m_groupIndex=m;
			}
		}
			break;

		case SurfaceAccessibleI::e_primitiveGroup:
		{
#ifdef FE_HOUDINI_USE_GA
			GA_PrimitiveGroup& rPrimitiveGroup=*(GA_PrimitiveGroup*)m_pGroup;
#else
			GB_PrimitiveGroup& rPrimitiveGroup=*(GB_PrimitiveGroup*)m_pGroup;
#endif
			if(!m_pPrimitive)
			{
				m_groupIndex=0;
				m_pPrimitive=m_pGdp->primitives().head(rPrimitiveGroup);
			}
			U32 m=m_groupIndex;
			for(;m>index && m_pPrimitive;m--)
			{
				m_pPrimitive=
						m_pGdp->primitives().prev(m_pPrimitive,rPrimitiveGroup);
			}
			for(;m<index && m_pPrimitive;m++)
			{
				m_pPrimitive=
						m_pGdp->primitives().next(m_pPrimitive,rPrimitiveGroup);
			}
			m_groupIndex=m;
		}
			break;

//		case SurfaceAccessibleI::e_vertex:
//			break;

//		case SurfaceAccessibleI::e_detail:
//			break;

		default:
			;
	}
}

} // namespace
