/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "math.pmh"

namespace fe
{

SpatialTransform operator*(const SpatialTransform& left,
		const SpatialTransform& right)
{
	// [i][j]=sum of [i][k]*[k][j] for all k

	SpatialTransform matrix;

	set(matrix.column(0),
			left(0,0)*right(0,0)+left(1,0)*right(0,1)+left(2,0)*right(0,2),
			left(0,0)*right(1,0)+left(1,0)*right(1,1)+left(2,0)*right(1,2),
			left(0,0)*right(2,0)+left(1,0)*right(2,1)+left(2,0)*right(2,2));

	set(matrix.column(1),
			left(0,1)*right(0,0)+left(1,1)*right(0,1)+left(2,1)*right(0,2),
			left(0,1)*right(1,0)+left(1,1)*right(1,1)+left(2,1)*right(1,2),
			left(0,1)*right(2,0)+left(1,1)*right(2,1)+left(2,1)*right(2,2));

	set(matrix.column(2),
			left(0,2)*right(0,0)+left(1,2)*right(0,1)+left(2,2)*right(0,2),
			left(0,2)*right(1,0)+left(1,2)*right(1,1)+left(2,2)*right(1,2),
			left(0,2)*right(2,0)+left(1,2)*right(2,1)+left(2,2)*right(2,2));

	set(matrix.column(3),
			left(0,3)*right(0,0)+left(1,3)*right(0,1)+left(2,3)*right(0,2)+
			right(0,3),
			left(0,3)*right(1,0)+left(1,3)*right(1,1)+left(2,3)*right(1,2)+
			right(1,3),
			left(0,3)*right(2,0)+left(1,3)*right(2,1)+left(2,3)*right(2,2)+
			right(2,3));

	return matrix;
}

FE_DL_PUBLIC BWORD makeFrame(SpatialTransform &a_frame,
	const SpatialVector &a_0,
	const SpatialVector &a_1, const SpatialVector &a_2,
	const SpatialVector &a_offset)
{
	SpatialVector direction = a_1 - a_0;
	SpatialVector up = a_2 - a_0;

	normalize(direction);
	normalize(up);

	SpatialVector side = cross(up, direction);
	if(isZero(side))
	{
		return FALSE;
	}
	normalize(side);
	up = cross(direction, side);
	if(isZero(up))
	{
		return FALSE;
	}
	normalize(up);

	a_frame.direction() = direction;
	a_frame.left() = side;
	a_frame.up() = up;
	a_frame.translation() = a_0;


	SpatialTransform offset_xform;
	setIdentity(offset_xform);
	translate(offset_xform, a_offset);
	a_frame *= offset_xform;

	return TRUE;
}

FE_DL_PUBLIC BWORD makeFrame(SpatialTransform &a_frame,
	const SpatialVector &a_0,
	const SpatialVector &a_1, const SpatialVector &a_2,
	const SpatialTransform &a_transform)
{
	SpatialVector direction = a_1 - a_0;
	SpatialVector up = a_2 - a_0;

	normalize(direction);
	normalize(up);

	SpatialVector side = cross(up, direction);
	if(isZero(side))
	{
		return FALSE;
	}
	normalize(side);
	up = cross(direction, side);
	if(isZero(up))
	{
		return FALSE;
	}
	normalize(up);

	a_frame.direction() = direction;
	a_frame.left() = side;
	a_frame.up() = up;
	a_frame.translation() = a_0;

	a_frame *= a_transform;
	return TRUE;
}

FE_DL_PUBLIC BWORD makeFrame(SpatialTransform &a_frame,
	const SpatialVector &a_location,
	const SpatialVector &a_direction, const SpatialVector &a_up)
{
	SpatialVector side = cross(a_up, a_direction);
	if(isZero(side))
	{
		return FALSE;
	}
	normalize(side);
	SpatialVector up = cross(a_direction, side);
	if(isZero(up))
	{
		return FALSE;
	}
	normalize(up);

	a_frame.direction() = a_direction;
	a_frame.left() = side;
	a_frame.up() = up;
	a_frame.translation() = a_location;
	return TRUE;
}

FE_DL_PUBLIC BWORD makeFrameNormalY(SpatialTransform &a_frame,
	const SpatialVector &a_location,
	const SpatialVector &a_tangent, const SpatialVector &a_normal)
{
	SpatialVector sideZ = cross(a_tangent,a_normal);
	if(isZero(sideZ))
	{
		return FALSE;
	}
	normalize(sideZ);
	SpatialVector tangentX = cross(a_normal, sideZ);
	if(isZero(tangentX))
	{
		return FALSE;
	}
	normalize(tangentX);

	a_frame.direction() = tangentX;
	a_frame.left() = a_normal;
	a_frame.up() = sideZ;
	a_frame.translation() = a_location;
	return TRUE;
}

} // namespace
