/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#define FE_STM_DEBUG	FALSE

#include "maya.pmh"

namespace fe
{

SurfaceTrianglesMaya::SurfaceTrianglesMaya(void)
{
#if FE_STM_DEBUG
	feLog("SurfaceTrianglesMaya()\n");
#endif
}

SurfaceTrianglesMaya::~SurfaceTrianglesMaya(void)
{
}

Protectable* SurfaceTrianglesMaya::clone(Protectable* pInstance)
{
#if FE_STM_DEBUG
	feLog("SurfaceTrianglesMaya::clone\n");
#endif

	SurfaceTrianglesMaya* pSurfaceTrianglesMaya= pInstance?
			dynamic_cast<SurfaceTrianglesMaya*>(pInstance):
			new SurfaceTrianglesMaya();

	SurfaceTriangles::clone(pSurfaceTrianglesMaya);

	// TODO copy attributes

	return pSurfaceTrianglesMaya;
}

void SurfaceTrianglesMaya::cache(void)
{
#if FE_STM_DEBUG
	feLog("SurfaceTrianglesMaya::cache\n");
#endif

	//* NOTE not doing any Record serialization

	//* convert Maya to vertex and normal arrays

	MFnMesh* pFnMesh=new MFnMesh(m_meshObject);

	const U32 count=pFnMesh->numPolygons();

	m_vertices=count*3;
	m_pVertexArray=(SpatialVector*)reallocate(m_pVertexArray,
			(m_vertices+1)*sizeof(SpatialVector));
	m_pNormalArray=(SpatialVector*)reallocate(m_pNormalArray,
			(m_vertices+1)*sizeof(SpatialVector));

	U32 total=0;
	for(U32 index=0;index<count;index++)
	{
		const U32 subCount=pFnMesh->polygonVertexCount(index);
		if(subCount!=3)
		{
#if FE_STM_DEBUG
			feLog("SurfaceTrianglesMaya::cache"
					" primitive %d/%d subCount=%d\n",index,count,subCount);
#endif
			continue;
		}

		U32 subs=pFnMesh->polygonVertexCount(index);

		MIntArray vertexArray;
		MStatus status=pFnMesh->getPolygonVertices(index,vertexArray);

		MFloatVectorArray normalArray;
		status=pFnMesh->getFaceVertexNormals(index,normalArray);
		I32 normalCount=normalArray.length();

		for(U32 subIndex=0;subIndex<subCount;subIndex++)
		{
			FEASSERT(subIndex<subs);
			if(subIndex>=subs)
			{
				continue;
			}

			MPoint point;
			status=pFnMesh->getPoint(vertexArray[subIndex],point);

			m_pVertexArray[total]=SpatialVector(point[0],point[1],point[2]);

			if(subIndex<normalCount)
			{
				MFloatVector& rNormal=normalArray[subIndex];
				m_pNormalArray[total]=
						SpatialVector(rNormal[0],rNormal[1],rNormal[2]);
			}
			else
			{
				m_pNormalArray[total]=SpatialVector(0.0,1.0,0.0);
			}

			total++;
		}
	}

	delete pFnMesh;

	if(total!=m_vertices)
	{
		feLog("SurfaceTrianglesMaya::cache"
				" converted %d/%d vertices\n",total,m_vertices);
	}

	calcBoundingSphere();
}

} // namespace
