/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __operator_MetaPlugin_h__
#define __operator_MetaPlugin_h__

namespace fe
{

/**************************************************************************//**
	@brief Common functionality of meta plugins for Houdini, Maya, etc

	@ingroup operator
*//***************************************************************************/
class FE_DL_EXPORT MetaPlugin
{
	public:
								MetaPlugin(void);
virtual							~MetaPlugin(void);

		void					setupSignaler(sp<Scope> a_spScope);
		void					setupBrush(sp<Scope> a_spScope);
		void					insertHandler(sp<HandlerI> a_spHandlerI);
		void					removeHandler(sp<HandlerI> a_spHandlerI);

	protected:
		void					clearReferences(void);

		sp<OperatorSurfaceI>			m_spOperatorSurfaceI;
		sp<SignalerI>					m_spSignalerI;
		sp<Layout>						m_spCookLayout;
		sp<Layout>						m_spBrushLayout;
		sp<DrawI>						m_spDrawOutput;
		sp<DrawI>						m_spDrawGuideImmediate;
		sp<DrawI>						m_spDrawGuideCached;
		sp<DrawI>						m_spDrawBrush;

		Record							m_cookSignal;
		Record							m_brushSignal;

		Accessor< Real >				m_aFrame;
		Accessor< Real >				m_aTime;
		Accessor< sp<Component> >		m_aSurfaceOutput;
		Accessor< sp<Component> >		m_aDrawOutput;
		Accessor< sp<Component> >		m_aDrawGuide;
		Accessor< sp<Component> >		m_aDrawBrush;
		Accessor<Record>				m_aWindowEvent;
		Accessor<SpatialTransform>		m_aCameraTransform;
		Accessor<SpatialVector>			m_aRayOrigin;
		Accessor<SpatialVector>			m_aRayDirection;

		std::vector< sp<Component> >	m_surfaceInputArray;
		sp<Component>					m_spSurfaceOutput;
		String							m_nameOutput;
		String							m_lastName;
		Real							m_lastTime;
};

} // namespace

#endif /* __operator_MetaPlugin_h__ */
